#include <QtOpenGL>
#include <algorithm>
#include <QtDebug>
#include "balken3ddiagram.h"

Balken3dDiagram::Balken3dDiagram(QWidget *parent)
         : QGLWidget(parent)
{
	xRot = -30*16;
	yRot = 155*16;
	zRot = -10*16;
}

void Balken3dDiagram::initializeGL()
{
         // Set up the rendering context, define display lists etc.:
         glClearColor(1.0, 1.0, 1.0, 0.0);
         glEnable(GL_DEPTH_TEST);
	 list = glGenLists(1);
	 glNewList(list, GL_COMPILE);
	 glBegin(GL_QUADS);
	 
	 //qglColor(Qt::green);
         glVertex3f(0.0,0.0,0.0);
	 glVertex3f(1.0,0.0,0.0);
	 glVertex3f(1.0,1.0,0.0);
         glVertex3f(0.0,1.0,0.0);

         glVertex3f(0.0,0.0,1.0);
	 glVertex3f(1.0,0.0,1.0);
	 glVertex3f(1.0,1.0,1.0);
         glVertex3f(0.0,1.0,1.0);

	 //qglColor(Qt::yellow);
	 glVertex3f(0.0,0.0,1.0);
	 glVertex3f(0.0,0.0,0.0);
	 glVertex3f(0.0,1.0,0.0);
         glVertex3f(0.0,1.0,1.0);

	 glVertex3f(1.0,0.0,1.0);
	 glVertex3f(1.0,0.0,0.0);
	 glVertex3f(1.0,1.0,0.0);
         glVertex3f(1.0,1.0,1.0);

	 //qglColor(Qt::blue);
	 glVertex3f(0.0,0.0,1.0);
	 glVertex3f(0.0,0.0,0.0);
	 glVertex3f(1.0,0.0,0.0);
         glVertex3f(1.0,0.0,1.0);

	 glVertex3f(0.0,1.0,1.0);
	 glVertex3f(0.0,1.0,0.0);
	 glVertex3f(1.0,1.0,0.0);
         glVertex3f(1.0,1.0,1.0);
         glEnd();
	 glEndList();
}

void Balken3dDiagram::resizeGL(int w, int h)
{
         // setup viewport, projection etc.:
         glViewport(0, 0, (GLint)w, (GLint)h);

 	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	//glOrtho(-20.0, +20.0, +20.0, -20.0, 10.0, 150.0);
	glOrtho(-1.5*cols, +1.5*cols, +20.0, -20.0, 10.0, 150.0);
	//glFrustum(-10.0, +10.0, +10.0, -10.0, 1.0, 150.0);
	glMatrixMode(GL_MODELVIEW);
}

void Balken3dDiagram::paintGL()
{
	 const double max = *(std::max_element(data.begin(),data.end()));

         glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	 glLoadIdentity();

	 glTranslated(0.0, 6.0, -110.0);
	 glRotated(xRot / 16.0, 1.0, 0.0, 0.0);
	 glRotated(yRot / 16.0, 0.0, 1.0, 0.0);
	 glRotated(zRot / 16.0, 0.0, 0.0, 1.0);

	 glBegin(GL_QUADS);
	 qglColor(Qt::gray);
         glVertex3f(2.0*cols/2+1.0,0.0,-2.0*rows/2);
	 glVertex3f(-2.0*cols/2,0.0,-2.0*rows/2);
	 glVertex3f(-2.0*cols/2,0.0,2.0*rows/2+1.0);
         glVertex3f(2.0*cols/2+1.0,0.0,2.0*rows/2+1.0);
	 glEnd();

	 glTranslated(-2.0*(cols/2),0.0,-2.0*(rows/2));
	for(int j=0;j<rows;++j)
	{
		for(int i = 0; i<cols;++i)
		{
			const double maxh = 25.0;
			const double d = data.at(i+cols*j)*maxh/max;
			if(d!=0.0)
			{
				if(d<0.1*maxh) qglColor(Qt::gray);
				else if(d<0.3*maxh) qglColor(Qt::lightGray);
				else if(d<0.5*maxh) qglColor(Qt::yellow);
				else if(d<0.7*maxh) qglColor(Qt::blue);
				else qglColor(Qt::green);
				glScaled(1.0,-d,1.0);
				glCallList(list);
				glScaled(1.0,-1.0/d,1.0);
			}
			glTranslated(2.0,0.0,0.0);
		}
		glTranslated(-2.0*cols,0.0,2.0);
	}
}

void Balken3dDiagram::mousePressEvent(QMouseEvent *event)
{
     lastPos = event->pos();
}

void Balken3dDiagram::mouseMoveEvent(QMouseEvent *event)
{
     int dx = event->x() - lastPos.x();
     int dy = event->y() - lastPos.y();

     if (event->buttons() & Qt::LeftButton) {
         xRot =  (xRot + 8 * dy);
         yRot =  (yRot + 8 * dx);
	 updateGL();
     } else if (event->buttons() & Qt::RightButton) {
	 xRot =  (xRot + 8 * dy);
	 zRot =  (zRot + 8 * dx);
	 updateGL();
     }
     lastPos = event->pos();
}

void Balken3dDiagram::setRowCount(int r)
{
	rows = r;
	data = QVector<double>(rows*rows);
}

int Balken3dDiagram::rowCount()
{
	return rows;
}

void Balken3dDiagram::setColumnCount(int c)
{
	cols = c;
	data = QVector<double>(rows*cols);
}

int Balken3dDiagram::columnCount()
{
	return cols;
}

void Balken3dDiagram::setData(int y,int x,double d)
{
	if(x >= cols || y >= rows || x<0 || y<0 ) return ;
	data[x+cols*y]=d;
}
