//
// C++ Implementation: xyplotter
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "xyplotter.h"

#include <QPainter>
#include <QVector>
#include <QStandardItemModel>
#include <QtDebug>
#include <QDrag>
#include <QMimeData>
#include <QApplication>
#include <numeric>
#include <algorithm>

XYPlotter::XYPlotter(QWidget* parent): ModelClient(parent),minX(0),minY(0),maxX(0),maxY(0)
{
	
}


XYPlotter::~XYPlotter()
{
}

void XYPlotter::setModel(QStandardItemModel *m)
{
	if(_model) _model->disconnect(this,SLOT(updateData()));
	_model = m;
	connect(m,SIGNAL(dataChanged( const QModelIndex&, const QModelIndex& )),this,SLOT(updateData()));
	updateData();
}

void XYPlotter::setSpalten(int x,int y)
{
	SpalteY=y;
	SpalteX=x;
	updateData();
}

void XYPlotter::updateData()
{
	if(!_model) return;
	
	// Alte Daten loeschen
	sx.clear();
	sy.clear();
	
	// Neue Daten einlesen
	for(int i=0;i<_model->rowCount();++i)
	{
		double cx = _model->data(_model->index(i,SpalteX)).toDouble();
		double cy = _model->data(_model->index(i,SpalteY)).toDouble();
		sx << cx;
		sy << cy;
	}
	minX = *std::min_element(sx.begin(),sx.end());
	minY = *std::min_element(sy.begin(),sy.end());
	maxX = *std::max_element(sx.begin(),sx.end());
	maxY = *std::max_element(sy.begin(),sy.end());
	update();
}

void XYPlotter::paintEvent( QPaintEvent *ev)
{
	const double wx = maxX-minX;
	const double wy = maxY-minY;
	
	QPainter p(this);
	
	scX = (width()-20)/wx;
	scY = (height()-20)/wy;
	
	p.fillRect(rect(),Qt::white);
	// Achsen
	p.drawLine(15,5,15,height()-15);
	p.drawLine(15,height()-15,width()-5,height()-15);
	p.drawLine(15,5,10,10);p.drawLine(15,5,20,10);
	p.drawLine(width()-5,height()-15,width()-10,height()-10);p.drawLine(width()-5,height()-15,width()-10,height()-20);
	
	p.setPen(Qt::blue);
	for(int i=0;i<sx.count();++i)
	{
		const double cx = 15+(sx[i]-minX)*scX,cy= height()-(sy[i]-minY)*scY-15;
		p.drawLine(cx-2,cy,cx+2,cy);
		p.drawLine(cx,cy-2,cx,cy+2);
	}
	
	paint(p); // Zusaetze
}

void XYPlotter::paint( QPainter &p)
{
	
}

// Drag and Drop
 void XYPlotter::mousePressEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton)
         dragStartPosition = event->pos();
 }


 void XYPlotter::mouseMoveEvent(QMouseEvent *event)
 {
     if (!(event->buttons() & Qt::LeftButton))
         return;
     if ((event->pos() - dragStartPosition).manhattanLength()
          < QApplication::startDragDistance())
         return;

     QDrag *drag = new QDrag(this);
     QMimeData *mimeData = new QMimeData;

     // Get Image
     QPixmap px = QPixmap::grabWidget(this);
     mimeData->setImageData(px.toImage());
     //mimeData->setData(mimeType, data);
     drag->setMimeData(mimeData);

     Qt::DropAction dropAction = drag->start(Qt::CopyAction | Qt::MoveAction);
 }





